"""Async executor versions of file functions from the os module."""
import asyncio
import sys
from functools import partial, wraps
import os


@asyncio.coroutine
@wraps(os.stat)
def stat(path, *, dir_fd=None, follow_symlinks=True, loop=None, executor=None):
    if loop is None:
        loop = asyncio.get_event_loop()
    fun = partial(os.stat, path, dir_fd=dir_fd, follow_symlinks=follow_symlinks)
    return loop.run_in_executor(executor, fun)


if 'posix' in sys.builtin_module_names:
    @asyncio.coroutine
    @wraps(os.sendfile)
    def sendfile(out, in_fd, offset, nbytes, *, loop=None, executor=None):
        if loop is None:
            loop = asyncio.get_event_loop()
        fun = partial(os.sendfile, out, in_fd, offset, nbytes)
        return loop.run_in_executor(executor, fun)


from .utils import (delegate_to_executor, proxy_property_directly,
                    proxy_method_directly)
from ..base import AsyncBase
from .._compat import PY_35


@delegate_to_executor('close', 'flush', 'isatty', 'read', 'readable',
                      'readline', 'readlines', 'seek', 'seekable', 'tell',
                      'truncate', 'write', 'writable', 'writelines')
@proxy_method_directly('detach', 'fileno', 'readable')
@proxy_property_directly('buffer', 'closed', 'encoding', 'errors',
                         'line_buffering', 'newlines')
class AsyncTextIOWrapper(AsyncBase):
    """The asyncio executor version of io.TextIOWrapper."""



"""Handle files using a thread pool executor."""
import asyncio

from io import (FileIO, TextIOBase, BufferedReader, BufferedWriter,
                BufferedRandom)
from functools import partial
from gzip import open as gzip_open, GzipFile
from zipfile import ZipFile, ZipExtFile, ZIP_STORED

from .binary import AsyncBufferedIOBase, AsyncBufferedReader, AsyncFileIO
from .text import AsyncTextIOWrapper
from ..base import AiofilesContextManager
from .._compat import singledispatch, PY_35

_sync_open = open
_sync_gzip_open = gzip_open

def _sync_zip_open(filename, mode, compression, allowZip64):
    zfile = ZipFile(filename, mode, compression, allowZip64)
    return zfile.open(zfile.namelist()[0])


__all__ = ('open', 'gzip_open', 'zip_open')


def open(file, mode='r', buffering=-1, encoding=None, errors=None, newline=None,
          closefd=True, opener=None, *, loop=None, executor=None):
    return AiofilesContextManager(_open(file, mode=mode, buffering=buffering,
                                        encoding=encoding, errors=errors,
                                        newline=newline, closefd=closefd,
                                        opener=opener, loop=loop,
                                        executor=executor))


@asyncio.coroutine
def _open(file, mode='r', buffering=-1, encoding=None, errors=None, newline=None,
          closefd=True, opener=None, *, loop=None, executor=None):
    """Open an asyncio file."""
    if loop is None:
        loop = asyncio.get_event_loop()
    cb = partial(_sync_open, file, mode=mode, buffering=buffering,
                 encoding=encoding, errors=errors, newline=newline,
                 closefd=closefd, opener=opener)
    f = yield from loop.run_in_executor(executor, cb)

    return wrap(f, loop=loop, executor=executor)


def gzip_open(filename, mode='rb', compresslevel=9, encoding=None,
          errors=None, newline=None, *, loop=None, executor=None):
    return AiofilesContextManager(_gzip_open(filename, mode=mode,
                                         compresslevel=compresslevel,
                                         encoding=encoding, errors=errors,
                                         newline=newline))


@asyncio.coroutine
def _gzip_open(filename, mode='rb', compresslevel=9, encoding=None,
           errors=None, newline=None, *, loop=None, executor=None):
    """Open an asyncio gzip file."""
    if loop is None:
        loop = asyncio.get_event_loop()

    cb = partial(_sync_gzip_open, filename, mode=mode, compresslevel=compresslevel,
                 encoding=encoding, errors=errors, newline=newline)
    f = yield from loop.run_in_executor(executor, cb)

    return wrap(f, loop=loop, executor=executor)


def zip_open(filename, mode='r', compression=ZIP_STORED, allowZip64=True, *,
             loop=None, executor=None):
    return AiofilesContextManager(
        _zip_open(
            filename, mode=mode, compression=compression,
            allowZip64=allowZip64,
            loop=loop, executor=executor
        )
    )


@asyncio.coroutine
def _zip_open(filename, mode='r', compression=ZIP_STORED, allowZip64=True, *,
              loop=None, executor=None):
    """Open an asyncio zip file."""
    if loop is None:
        loop = asyncio.get_event_loop()

    cb = partial(_sync_zip_open, filename, mode=mode,
                 compression=compression, allowZip64=allowZip64)
    f = yield from loop.run_in_executor(executor, cb)

    return wrap(f, loop=loop, executor=executor)


@singledispatch
def wrap(file, *, loop=None, executor=None):
    raise TypeError('Unsupported io type: {}.'.format(file))


@wrap.register(TextIOBase)
def _(file, *, loop=None, executor=None):
    return AsyncTextIOWrapper(file, loop=loop, executor=executor)


@wrap.register(BufferedWriter)
def _(file, *, loop=None, executor=None):
    return AsyncBufferedIOBase(file, loop=loop, executor=executor)


@wrap.register(BufferedReader)
@wrap.register(BufferedRandom)
@wrap.register(GzipFile)
@wrap.register(ZipExtFile)
def _(file, *, loop=None, executor=None):
    return AsyncBufferedReader(file, loop=loop, executor=executor)


@wrap.register(FileIO)
def _(file, *, loop=None, executor=None):
    return AsyncFileIO(file, loop, executor)




import codecs
import os
import re
from setuptools import setup, find_packages

with open('README.rst') as f:
    readme = f.read()


def read(*parts):
    here = os.path.abspath(os.path.dirname(__file__))
    return codecs.open(os.path.join(here, *parts), 'r').read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)

    raise RuntimeError("Unable to find version string.")

setup(
    name='aiofiles-ext',
    version=find_version('aiofiles', '__init__.py'),
    packages=find_packages(),
    url='https://github.com/Tinche/aiofiles',
    license='Apache 2.0',
    author='Tin Tvrtkovic',
    author_email='tinchester@gmail.com',
    description='File support for asyncio.',
    long_description=readme,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Topic :: System :: Filesystems",
    ],
    extras_require={
        ':python_version == "3.3"': ['asyncio', 'singledispatch']
    }
)
